@@ -6,7 +6,7 @@
 ;; $Id$
 
 ;; This file is part of GNU Anubis.
-;; Copyright (C) 2003-2014 The Anubis Team.
+;; Copyright (C) 2003-2024 The Anubis Team.
 
 ;; GNU Anubis is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by the
@@ -143,6 +143,8 @@
 	      "esmtp-passcode"
 	      "esmtp-realm"
 	      "esmtp-anonymous-token"
+	      "identd-keyfile"
+	      "use-pam"
 	      "mode"
 	      "smtp-greeting-message"
 	      "smtp-help-message"
