@@ -2,7 +2,7 @@
    guile.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2003-2020 The Anubis Team.
+   Copyright (C) 2003-2024 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -22,8 +22,6 @@
 #include "extern.h"
 #include "rcfile.h"
 
-#ifdef WITH_GUILE
-
 static void guile_ports_open (void);
 static void guile_ports_close (void);
 
@@ -523,6 +521,4 @@ guile_section_init (void)
   rc_secdef_add_child (sp, &guile_rule_secdef_child);
 }
 
-#endif /* WITH_GUILE */
-
 /* EOF */
