@@ -2,7 +2,7 @@
    mime.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001-2014 The Anubis Team.
+   Copyright (C) 2001-2024 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -96,7 +96,7 @@ message_append_signature_file (MESSAGE msg)
 
   n = strlen (homedir) + strlen (signature_file) + 2;
   signature_path = xmalloc (n);
-  snprintf (signature_path, n - 1, "%s/%s", homedir, signature_file);
+  snprintf (signature_path, n, "%s/%s", homedir, signature_file);
 
   message_append_text_file (msg, signature_path, "-- \n");
   free (signature_path);
