@@ -2,7 +2,7 @@
    pgsql.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2004-2014 The Anubis Team.
+   Copyright (C) 2004-2024 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -24,7 +24,7 @@
 #ifdef WITH_PGSQL
 
 #include "sql.h"
-#include <libpq-fe.h>
+#include <postgresql/libpq-fe.h>
 
 /* PostgreSQL URL:
 
